/**
 * Copyright (c) 2018 开源 All rights reserved.
 * <p>
 * https://www.link.io
 * <p>
 * 版权所有，侵权必究！
 */

package com.link.modules.sys.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.link.common.constant.Constant;
import com.link.common.exception.ErrorCode;
import com.link.common.exception.RenException;
import com.link.common.page.PageData;
import com.link.common.service.impl.BaseServiceImpl;
import com.link.common.utils.ConvertUtils;
import com.link.common.utils.Result;
import com.link.modules.security.user.SecurityUser;
import com.link.modules.security.user.UserDetail;
import com.link.modules.sys.dao.SysUserDao;
import com.link.modules.sys.dto.SysDeptDTO;
import com.link.modules.sys.dto.SysDictDataDTO;
import com.link.modules.sys.dto.SysUserDTO;
import com.link.modules.sys.entity.SysUserEntity;
import com.link.modules.security.password.PasswordUtils;
import com.link.modules.sys.enums.SuperAdminEnum;
import com.link.modules.sys.enums.UserStatusEnum;
import com.link.modules.sys.service.SysDeptService;
import com.link.modules.sys.service.SysDictDataService;
import com.link.modules.sys.service.SysRoleUserService;
import com.link.modules.sys.service.SysUserService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;


/**
 * 系统用户
 *
 * @author Mark sunlightcs@gmail.com
 */
@Service
public class SysUserServiceImpl extends BaseServiceImpl<SysUserDao, SysUserEntity> implements SysUserService {
    @Autowired
    private SysRoleUserService sysRoleUserService;
    @Autowired
    private SysDeptService sysDeptService;
    @Autowired
    private SysDictDataService sysDictService;

    @Override
    public PageData<SysUserDTO> page(Map<String, Object> params) {
        //转换成like
        paramsToLike(params, "username");

        //分页
        IPage<SysUserEntity> page = getPage(params, Constant.CREATE_DATE, false);

        //普通管理员，只能查询所属部门及子部门的数据
        UserDetail user = SecurityUser.getUser();
        if (user.getSuperAdmin() == SuperAdminEnum.NO.value()) {
            params.put("deptIdList", sysDeptService.getSubDeptIdList(user.getDeptId()));
        }

        //查询
        List<SysUserEntity> list = baseDao.getList(params);

        return getPageData(list, page.getTotal(), SysUserDTO.class);
    }

    @Override
    public List<SysUserDTO> list(Map<String, Object> params) {
        //普通管理员，只能查询所属部门及子部门的数据
        UserDetail user = SecurityUser.getUser();
        if (user.getSuperAdmin() == SuperAdminEnum.NO.value()) {
            params.put("deptIdList", sysDeptService.getSubDeptIdList(user.getDeptId()));
        }

        List<SysUserEntity> entityList = baseDao.getList(params);

        return ConvertUtils.sourceToTarget(entityList, SysUserDTO.class);
    }

    @Override
    public SysUserDTO get(Long id) {
        SysUserEntity entity = baseDao.getById(id);

        return ConvertUtils.sourceToTarget(entity, SysUserDTO.class);
    }

    @Override
    public SysUserDTO getByUsername(String username) {
        SysUserEntity entity = baseDao.getByUsername(username);
        return ConvertUtils.sourceToTarget(entity, SysUserDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(SysUserDTO dto) {
        SysUserEntity entity = ConvertUtils.sourceToTarget(dto, SysUserEntity.class);

        //密码加密
        String password = PasswordUtils.encode(entity.getPassword());
        entity.setPassword(password);

        //保存用户
        entity.setSuperAdmin(SuperAdminEnum.NO.value());
        insert(entity);

        //保存角色用户关系
        sysRoleUserService.saveOrUpdate(entity.getId(), dto.getRoleIdList());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(SysUserDTO dto) {
        SysUserEntity entity = ConvertUtils.sourceToTarget(dto, SysUserEntity.class);

        //密码加密
        if (StringUtils.isBlank(dto.getPassword())) {
            entity.setPassword(null);
        } else {
            String password = PasswordUtils.encode(entity.getPassword());
            entity.setPassword(password);
        }

        //更新用户
        updateById(entity);

        //更新角色用户关系
        sysRoleUserService.saveOrUpdate(entity.getId(), dto.getRoleIdList());
    }

    @Override
    public void delete(Long[] ids) {
        //删除用户
        baseDao.deleteBatchIds(Arrays.asList(ids));

        //删除角色用户关系
        sysRoleUserService.deleteByUserIds(ids);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePassword(Long id, String newPassword) {
        newPassword = PasswordUtils.encode(newPassword);

        baseDao.updatePassword(id, newPassword);
    }

    @Override
    public int getCountByDeptId(Long deptId) {
        return baseDao.getCountByDeptId(deptId);
    }

    @Override
    public List<Long> getUserIdListByDeptId(List<Long> deptIdList) {
        return baseDao.getUserIdListByDeptId(deptIdList);
    }

    @Override
    public void importExcel(MultipartFile file) {
        String suffix = StringUtils.substringAfterLast(file.getOriginalFilename(), ".");
        if (StringUtils.equalsIgnoreCase(suffix, "xlsx") || StringUtils.equalsIgnoreCase(suffix, "xls")) {
            try {
                ExcelReader reader = ExcelUtil.getReader(file.getInputStream());
                Map<String, String> headerAlias = new HashMap<>(5);
                // 设置Excel表头中文对应的Bean字段名称
                headerAlias.put("姓名", "realName");
                headerAlias.put("帐号", "username");
                headerAlias.put("部门", "deptName");
                headerAlias.put("性别", "genderName");
                headerAlias.put("手机", "mobile");
                headerAlias.put("个人邮箱", "email");

                reader.setHeaderAlias(headerAlias);
                List<SysUserDTO> sysUserDtoList = reader.read(8, 9, SysUserDTO.class);

                Map<String, Object> params = new HashMap<>();
                //获取部门
                List<SysDeptDTO> deptDtoList = sysDeptService.getJoinList(params);
                Map<String, Long> deptMap = deptDtoList.stream().collect(Collectors.toMap(sysDeptDTO -> {
                    return sysDeptDTO.getParentNames();
                }, SysDeptDTO::getId));

                //获取性别
                params.clear();
                params.put("dictType", "gender");
                List<SysDictDataDTO> sysDictDtoList = sysDictService.list(params);
                Map<String, String> genderMap =
                        sysDictDtoList.stream().collect(Collectors.toMap(SysDictDataDTO::getDictLabel,
                                SysDictDataDTO::getDictValue));

                List<String> usernameList = new ArrayList<>();

                //默认密码
                String password = PasswordUtils.encode("111111");
                if (CollectionUtils.isNotEmpty(sysUserDtoList)) {
                    for (SysUserDTO sysUserDTO : sysUserDtoList) {
                        if (usernameList.contains(sysUserDTO.getUsername())) {
                            throw new RenException("用户名{" + sysUserDTO.getUsername() + "}重复");
                        }
                        usernameList.add(sysUserDTO.getUsername());
                        sysUserDTO.setGender(Convert.toInt(genderMap.get(sysUserDTO.getGenderName())));
                        //获取部门 将微信中的部门分隔符换成我们内部使用的\
                        String deptName = sysUserDTO.getDeptName().replaceAll("/", "\\\\");
                        Long deptId = deptMap.get(deptName);
                        if (null == deptId) {
                            deptId = sysDeptService.insertNewDeptFromWx(deptName);
                        }
                        sysUserDTO.setDeptId(deptId);
                        sysUserDTO.setPassword(password);
                        sysUserDTO.setSuperAdmin(SuperAdminEnum.NO.value());
                        sysUserDTO.setStatus(UserStatusEnum.ENABLED.value());
                    }
                }
                //遍历导入的用户信息，对存在的用户进行更新，对不存在的进行创建，对删除的用户进行删除（软删除）
                params.clear();
                List<SysUserDTO> oldUserList = this.list(params);
                Map<String, SysUserDTO> oldUserMap =
                        oldUserList.stream().collect(Collectors.toMap(SysUserDTO::getUsername, user -> user));
                for (SysUserDTO newUser : sysUserDtoList) {
                    if (oldUserMap.containsKey(newUser.getUsername())) {
                        // 更新
                        SysUserDTO oldUser = oldUserMap.get(newUser.getUsername());
                        oldUser.setDeptId(newUser.getDeptId());
                        oldUser.setGender(newUser.getGender());
                        oldUser.setMobile(newUser.getMobile());
                        oldUser.setRealName(newUser.getRealName());
                        oldUser.setEmail(newUser.getEmail());
                        baseDao.updateById(ConvertUtils.sourceToTarget(oldUser, SysUserEntity.class));
                    } else {
                        //
                        this.insert(ConvertUtils.sourceToTarget(newUser, SysUserEntity.class));
                    }
                }
                if (oldUserMap.size() > 0) {
                    // 说明还有一些用户需要在此次更新过程中删除
                    List<Long> deleteUserIdList = new ArrayList<>();
                    oldUserMap.values().forEach(user -> deleteUserIdList.add(user.getId()));
                    baseDao.deleteBatchIds(deleteUserIdList);
                }
            } catch (IOException e) {
                throw new RenException(ErrorCode.ERR_USER_FILE_PARSE_ERR);
            }
        } else {
            throw new RenException(ErrorCode.ERR_BID_FILE_PATTERN_ERR);
        }
    }

}
