/**
 * Copyright (c) 2018 开源 All rights reserved.
 *
 * https://www.link.io
 *
 * 版权所有，侵权必究！
 */

package com.link.service.impl;

import com.link.common.exception.ErrorCode;
import com.link.common.exception.RenException;
import com.link.common.service.impl.BaseServiceImpl;
import com.link.common.validator.AssertUtils;
import com.link.dao.UserDao;
import com.link.entity.TokenEntity;
import com.link.entity.UserEntity;
import com.link.dto.LoginDTO;
import com.link.service.TokenService;
import com.link.service.UserService;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.HashMap;
import java.util.Map;

@Service
public class UserServiceImpl extends BaseServiceImpl<UserDao, UserEntity> implements UserService {
	@Autowired
	private TokenService tokenService;

	@Override
	public UserEntity getByEmail(String email) {
		return baseDao.getUserByEmail(email);
	}

	@Override
	public UserEntity getUserByEmail(String email) {
		return baseDao.getUserByEmail(email);
	}

	@Override
	public Map<String, Object> login(LoginDTO dto) {
		UserEntity user = getByEmail(dto.getEmail());
		AssertUtils.isNull(user, ErrorCode.ACCOUNT_PASSWORD_ERROR);

		//密码错误
		if(!user.getPassword().equals(getMD5(dto.getPassword()))){
			throw new RenException(ErrorCode.ACCOUNT_PASSWORD_ERROR);
		}

		//获取登录token
		TokenEntity tokenEntity = tokenService.createToken(user.getEmail());

		Map<String, Object> map = new HashMap<>(2);
		map.put("token", tokenEntity.getToken());
		map.put("expire", tokenEntity.getExpireDate().getTime() - System.currentTimeMillis());

		return map;
	}

	/**
	 * 对字符串做MD5加密
	 */
	private static String getMD5(String str) {
		try {
			// 生成一个MD5加密计算摘要
			MessageDigest md = MessageDigest.getInstance("MD5");
			// 计算md5函数
			md.update(str.getBytes());
			// digest()最后确定返回md5 hash值，返回值为8为字符串。因为md5 hash值是16位的hex值，实际上就是8位的字符
			// BigInteger函数则将8位的字符串转换成16位hex值，用字符串来表示；得到字符串形式的hash值
			return new BigInteger(1, md.digest()).toString(32);
		} catch (Exception e) {
			return null;
		}
	}

}